/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.activities;

import java.lang.ref.WeakReference;
import java.util.LinkedList;
import java.util.List;

import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.widget.ListView;

import com.danielme.muspyforandroid.Constants;
import com.danielme.muspyforandroid.R;
import com.danielme.muspyforandroid.activities.adapters.MyArtistsAdapter;
import com.danielme.muspyforandroid.activities.adapters.template.CustomListAdapter;
import com.danielme.muspyforandroid.activities.base.AbstractListActivity;
import com.danielme.muspyforandroid.model.Artist;
import com.danielme.muspyforandroid.services.MuspyClient;

/**
 * Manages artists followed by user.
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 *
 */
public class MyArtistsActivity extends AbstractListActivity
{

	/* ************************************************************ */
	/*							FIELDS            					*/
	/* ************************************************************ */

	private List<Artist> artists;

	private boolean checkAll = true;
	
	private ArtistsHandler artistsHandler = new ArtistsHandler(this);

	/* ************************************************************ */
	/*							OVERRIDE           					*/
	/* ************************************************************ */


	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.myartists);
		markButtonPressed(R.id.artists, false);
		
		getArtists(false);
	}

	@Override
	protected void onListItemClick(ListView l, View v, int position, long id)
	{
		Artist artist = (Artist) getCustomListAdapter().getItem(position);
		Intent intent = new Intent(this, ReleasesActivity.class);
		Bundle bundle = new Bundle();
		bundle.putString(Constants.PARAM_ARTIST, artist.getMbid());
		intent.putExtras(bundle); 
		startActivity(intent);
		overridePendingTransition(R.anim.leftin, R.anim.leftout);
	}

	/* ************************************************************ */
	/*							BUTTONS            					*/
	/* ************************************************************ */

	public void search(View view)
	{
		Intent intent = new Intent(this, SearchArtistsActivity.class);
		defaultTransition(intent);
	}

	private void delete(final List<String> checked)
	{
		if (checkConnection())
		{
			showProgressDialog(getString(R.string.sendingdelete));
			new Thread()
			{
				public void run()
				{
					try
					{
						MuspyClient.getInstance().deleteArtists(getApplicationContext(), checked);
						artists =  MuspyClient.getInstance().getArtists(MyArtistsActivity.this);
						artistsHandler.sendEmptyMessage(2);
					}
					catch (Exception ex)
					{
						Log.e(MyArtistsActivity.class.toString(), ex.getMessage(), ex);
						Message message = new Message();
						message.obj = ex;
						message.what = 1;
						artistsHandler.sendMessage(message);
					}
				}
			}.start();
		}
	}

	
	public void delete(View view)
	{
		final List<String> artistsChecked = getArtistsChecked();
		//no artists selected
		if (artistsChecked == null || artistsChecked.isEmpty())
		{
			Builder builder = createInfoDialog(getString(R.string.noartistchecked));			
			
			builder.setNeutralButton(getString(R.string.ok), null);			

			AlertDialog alertDialog = builder.create();
			alertDialog.show();
			alertDialog.setCancelable(false);
			dialogStyle(alertDialog);
		}
		// confirm dialog and
		else
		{			
			String msg = getString(R.string.deleteartist);
			if (artistsChecked.size() > 15)
			{
				msg += getString(R.string.awhile);
			}
			AlertDialog.Builder builder = createInfoDialog(msg);
			
			builder.setCancelable(false).setPositiveButton(R.string.yes, new DialogInterface.OnClickListener()
			{
				public void onClick(DialogInterface dialog, int id)
				{
					MyArtistsActivity.this.delete(artistsChecked);
				}
			}).setNegativeButton(R.string.no, new DialogInterface.OnClickListener()
			{
				public void onClick(DialogInterface dialog, int id)
				{
					dialog.cancel();
				}
			});
			AlertDialog alertDialog = builder.create();
			alertDialog.show();
			alertDialog.setCancelable(false);
			dialogStyle(alertDialog);
		}
	}

	public void importlastfm(View view)
	{
		Intent intent = new Intent(this, ImportLastfmActivity.class);
		defaultTransition(intent);
	}

	public void selectall(View view)
	{
		if (getCustomListAdapter() != null)
		{
			@SuppressWarnings("unchecked")
			List<Artist> artists = (List<Artist>) getCustomListAdapter().getObjects();
			for (Artist artist : artists)
			{
				artist.setChecked(checkAll);
			}
			checkAll = !checkAll;
			getCustomListAdapter().notifyDataSetChanged();
		}
	}	
	
	public void refresh(View view)
	{
		getArtists(true);
	}

	/* ************************************************************ */
	/*							PRIVATE            					*/
	/* ************************************************************ */

	
	private void getArtists(final boolean refresh)
	{
		if (checkConnection())
		{
			showProgressDialog(null);
			if (getCustomListAdapter() != null)  
			{
				getCustomListAdapter().clearData();
			}
			new Thread()
			{
				public void run()
				{
					try
					{
						if (refresh)
						{
							artists =  MuspyClient.getInstance().refreshArtists(MyArtistsActivity.this);
							artistsHandler.sendEmptyMessage(3);
						}
						else
						{
							artists =  MuspyClient.getInstance().getArtists(MyArtistsActivity.this);
							artistsHandler.sendEmptyMessage(0);
						}
						
					}
					catch (Exception ex)
					{
						Log.e(MyArtistsActivity.class.toString(), ex.getMessage(), ex);
						Message message = new Message();
						message.obj = ex;
						message.what = 1;
						artistsHandler.sendMessage(message);
					}
				}

			}.start();
		}
	}

	/**
	 * Updates artists.
	 */
	private static class ArtistsHandler extends Handler
	{
		
        private final WeakReference<MyArtistsActivity> myActivity;
        
        public ArtistsHandler(MyArtistsActivity myActivity)
         {
        	      this.myActivity = new WeakReference<MyArtistsActivity>(myActivity);
         }   
        
        private MyArtistsActivity getActivity()
        {
        	return myActivity.get();
        }
        
		@Override
		public void handleMessage(Message msg)
		{
			getActivity().getProgressDialog().dismiss();
			//error
			if (msg.what == 1)
			{
				getActivity().showError((Exception) msg.obj);
			}
			// delete, clear data
			else if (msg.what == 2)
			{	
				getActivity().clearData();
				getActivity().appendData(getActivity().artists);
				if (getActivity().artists.isEmpty())
				{
					showNoArtistsMessage();
				} 
				
				getActivity().updateDisplaying(R.string.displayArtists, null);
				getActivity().showToast(R.string.toastremoved);	
				
				
			}
			else if (msg.what == 3)
			{	
				getActivity().clearData();
				getActivity().appendData(getActivity().artists);
				if (getActivity().artists.isEmpty())
				{
					showNoArtistsMessage();
				}
				if (getActivity().getCustomListAdapter() != null)
				{
					getActivity().updateDisplaying(R.string.displayArtists, getActivity().artists.size() + "");
				}
				else
				{
					getActivity().setListAdapter(new CustomListAdapter(getActivity(), R.layout.myartists_listview, getActivity().artists, MyArtistsAdapter.class));
				}
			}
			else if (getActivity().artists.isEmpty())
			{				
				showNoArtistsMessage();
			}
			else
			{
				//first search
				if (getActivity().getCustomListAdapter() == null)
				{
					getActivity().setListAdapter(new CustomListAdapter(getActivity(), R.layout.myartists_listview, getActivity().artists, MyArtistsAdapter.class));
				}
				else
				{
					getActivity().appendData(getActivity().artists);
				}

				getActivity().updateDisplaying(R.string.displayArtists, null);
				getActivity().findViewById(R.id.noartistsLayout).setVisibility(View.GONE);

			}
			getActivity().checkAll = true;

		}

		private void showNoArtistsMessage()
		{
			getActivity().findViewById(R.id.displaying).setVisibility(View.GONE);
			getActivity().findViewById(R.id.noartistsLayout).setVisibility(View.VISIBLE);
			getActivity().getListView().setVisibility(View.GONE);
		}

	};


	/**
	 * Returns mbids of the artists selected.
	 */
	private List<String> getArtistsChecked()
	{
		List<String> checked = null;
		
		if (getCustomListAdapter() != null)
		{
			@SuppressWarnings("unchecked")
			List<Artist> artists = (List<Artist>) getCustomListAdapter().getObjectsUnmodifiable();
			 checked = new LinkedList<String>();
			for (Artist artist : artists)
			{
				if (artist.isChecked())
				{
					checked.add(artist.getMbid());
				}
			}			
		}
		return checked;
	}

}