/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.activities;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.AbsListView;
import android.widget.AbsListView.OnScrollListener;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.TextView.OnEditorActionListener;

import com.danielme.muspyforandroid.Constants;
import com.danielme.muspyforandroid.R;
import com.danielme.muspyforandroid.activities.adapters.SearchArtistAdapter;
import com.danielme.muspyforandroid.activities.adapters.template.CustomListAdapter;
import com.danielme.muspyforandroid.activities.base.AbstractListActivity;
import com.danielme.muspyforandroid.model.Artist;
import com.danielme.muspyforandroid.services.MusicBrainzClient;
import com.danielme.muspyforandroid.services.MuspyClient;
import com.danielme.muspyforandroid.utils.Utils;

/**
 * EndlessListView. Show artists from MusicBrainz.
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 */
public class SearchArtistsActivity extends AbstractListActivity
{

	/* ************************************************************ */
	/*							FIELDS            					*/
	/* ************************************************************ */

	private EditText searchEditText;	

	private View footerView;

	private boolean loading = false;

	private Artist artist;
	
	private SearchHandler searchHandler = new SearchHandler(this);
	
	private AddHandler addHandler = new AddHandler(this);

	/**
	 * Number of artists found.
	 */
	private StringBuilder total = new StringBuilder("0");


	/* ************************************************************ */
	/*							OVERRIDE          					*/
	/* ************************************************************ */

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.searchartists);	
		markButtonPressed(R.id.artists, true);
		searchEditText = (EditText) findViewById(R.id.searchtext);	
		//search button in keyboard
		searchEditText.setOnEditorActionListener(new OnEditorActionListener()
		{
			@Override
			public boolean onEditorAction(TextView textView, int actionId, KeyEvent event)
			{
				boolean action = false;
				if (actionId == EditorInfo.IME_ACTION_SEARCH)
				{
					InputMethodManager inputMethodManager = (InputMethodManager) textView.getContext().getSystemService(Context.INPUT_METHOD_SERVICE);
					inputMethodManager.hideSoftInputFromWindow(textView.getWindowToken(), 0);
					search(textView);
					action = true;
				}
				return action;
			}
		});		

		footerView = ((LayoutInflater) this.getSystemService(Context.LAYOUT_INFLATER_SERVICE)).inflate(R.layout.listviewfooter, null, false);

		getListView().addFooterView(footerView, null, false);
		setListAdapter(new CustomListAdapter(this, R.layout.searchartists_listview, new ArrayList<Artist>(0), SearchArtistAdapter.class));		
		getListView().removeFooterView(footerView);
		getListView().setOnScrollListener(new OnScrollListener()
		{
			@Override
			public void onScrollStateChanged(AbsListView view, int scrollState)
			{
				//not implemented
			}

			@Override
			public void onScroll(AbsListView view, int firstVisibleItem, int visibleItemCount, int totalItemCount)
			{				
				if (getCustomListAdapter() != null && getCount() > 0 && showLast(firstVisibleItem, visibleItemCount, totalItemCount) 
						&& getCount() < Integer.parseInt(total.toString()) && !loading && !isShowingNoConnection() && !isShowingError())
				{
					if (checkConnection())
					{
						loading = true;
						getListView().addFooterView(footerView);
						(new LoadNextPage()).execute("");
					}
					else
					{
						getListView().removeFooterView(footerView);
					}
				}
			}
		});
	}

	@Override
	protected void onListItemClick(ListView listView, View view, int position, long id)
	{
		Artist artist = (Artist) getCustomListAdapter().getItem(position);
		Intent intent = new Intent(this, ReleasesActivityFromSearch.class);
		Bundle bundle = new Bundle();
		bundle.putString(Constants.PARAM_ARTIST, artist.getMbid());
		bundle.putString(Constants.PARAM_ARTIST_NAME, artist.getName());
		intent.putExtras(bundle);
		startActivity(intent);
		overridePendingTransition(R.anim.leftin, R.anim.leftout);
	}

	/* ************************************************************ */
	/*							BUTTONS          					*/
	/* ************************************************************ */

	public void search(View view)
	{
		if (Utils.hasText(searchEditText) && checkConnection())
		{
			//hides keyboard
			InputMethodManager inputMethodManager = (InputMethodManager) getSystemService(INPUT_METHOD_SERVICE);
			inputMethodManager.hideSoftInputFromWindow(searchEditText.getWindowToken(), 0);
			showProgressDialog(getString(R.string.sendingdata));

			new Thread()
			{
				public void run()
				{
					try
					{
						List<Artist> artistsFromWS = MusicBrainzClient.getInstance().searchArtists(searchEditText.getText().toString(), 0, total, getListSize());							
						Message msg = new Message();
						msg.obj = artistsFromWS;
						msg.what = 0;
						searchHandler.sendMessage(msg);
						}
						catch (Exception ex)
						{
							Log.e(SearchArtistsActivity.class.toString(), ex.getMessage(), ex);
							Message message = new Message();
							message.obj = ex;
							message.what = 1;
							searchHandler.sendMessage(message);
						}
					}
			}.start();

		}
	}

	public void add(View view)
	{
		ImageView imageButton = (ImageView) view;
		artist = (Artist) imageButton.getTag();
		showProgressDialog(getString(R.string.sendingdata));

		new Thread()
		{
			public void run()
			{
				try
				{
					MuspyClient.getInstance().addArtist(SearchArtistsActivity.this, artist.getMbid());
					addHandler.sendEmptyMessage(0);
				}
				catch (Exception ex)
				{
					Log.e(SearchArtistsActivity.class.toString(), ex.getMessage(), ex);
					Message message = new Message();
					message.obj = ex;
					message.what = 1;
					searchHandler.sendMessage(message);
				}
			}
		}.start();
	}

	/* ************************************************************ */
	/*							PRIVATE          					*/
	/* ************************************************************ */

	private static class SearchHandler extends Handler
	{
		private final WeakReference<SearchArtistsActivity> myActivity;
        
        public SearchHandler(SearchArtistsActivity myActivity)
         {
        	      this.myActivity = new WeakReference<SearchArtistsActivity>(myActivity);
         }   
        
        private SearchArtistsActivity getActivity()
        {
        	return myActivity.get();
        }
		
		@SuppressWarnings("unchecked")
		@Override
		public void handleMessage(Message msg)
		{
			getActivity().getProgressDialog().dismiss();
			if (msg.what == 1)
			{
				getActivity().showError((Exception) msg.obj);
			}
			else
			{
				getActivity().clearData();
				getActivity().appendData((Collection<Artist>) msg.obj);
				getActivity().updateDisplaying(R.string.displaySearch, getActivity().total.toString());
				getActivity().getListView().setSelection(0);
			}
		} 
	};


	private class LoadNextPage extends AsyncTask<String, Void, String>
	{
		private List<Artist> artistsFromWS;
		
		private Exception exc = null;

		@Override
		protected String doInBackground(String... arg0)
		{
			String res = null;
			try 
			{
				artistsFromWS = MusicBrainzClient.getInstance().searchArtists(searchEditText.getText().toString(), getCount(), total, getListSize());
			}
			catch (Exception exc)
			{
				Log.e(SearchArtistsActivity.class.toString(), exc.getMessage(), exc);
				this.exc = exc;
			}

			return res;
		}
 
		@Override
		protected void onPostExecute(String result)
		{			
			if (exc != null)
			{
				showError(exc);				
			}
			else
			{				
				appendData(artistsFromWS);
				updateDisplaying(R.string.displaySearch, total.toString());				
			}	
			getListView().removeFooterView(footerView);
			loading = false;		
		}

	}

	private static class AddHandler extends Handler
	{
		
		private final WeakReference<SearchArtistsActivity> myActivity;
        
        public AddHandler(SearchArtistsActivity myActivity)
         {
        	      this.myActivity = new WeakReference<SearchArtistsActivity>(myActivity);
         }   
        
        private SearchArtistsActivity getActivity()
        {
        	return myActivity.get();
        }
        
		@Override
		public void handleMessage(Message msg)
		{
			getActivity().getProgressDialog().dismiss();
			if (msg.what == 1)
			{
				getActivity().showError((Exception) msg.obj);
			}
			else
			{				
				String  text = String.format(getActivity().getString(R.string.toastadded), getActivity().artist.getName());
				getActivity().showToastSuccess(text);
				Intent intent = new Intent(getActivity(), MyArtistsActivity.class);
				getActivity().defaultTransition(intent);
			}
		}
	};

}