/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.activities.adapters;

import java.util.List;
import java.util.Map;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.os.AsyncTask;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.danielme.muspyforandroid.Constants;
import com.danielme.muspyforandroid.R;
import com.danielme.muspyforandroid.activities.adapters.template.ICustomListAdapterHolderPattern;
import com.danielme.muspyforandroid.cache.CoverMemoryCache;
import com.danielme.muspyforandroid.model.Release;
import com.danielme.muspyforandroid.services.MuspyClient;

/**
 * Adapter for releases.
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 */
public class ReleasesAdapter implements ICustomListAdapterHolderPattern
{

	/* ************************************************************ */
	/*							FIELDS            					*/
	/* ************************************************************ */

	private TextView dateTextView;
	private TextView releaseTextView;
	private TextView artistTextView;
	private ImageView coverImageView;
	private String mbid;


	/* ************************************************************ */
	/*							OVERRIDE           					*/
	/* ************************************************************ */

	@Override
	public void populateHolder(View row, int position, List<?> objects, Context context, Map<?, ?> params)
	{
		Release release = (Release) objects.get(position);
		
		dateTextView.setText(release.getDateLocale());
		artistTextView.setText(release.getArtist().getName());
		mbid = release.getMbid();
		this.releaseTextView.setText("[" + release.getType() + "] " + release.getName());

		if ((Boolean) params.get(Constants.PREF_LOADCOVER))
		{
			coverImageView.setVisibility(View.VISIBLE);
			Drawable drawable = CoverMemoryCache.get(release.getMbid());

			if (drawable == null)
			{
				coverImageView.setImageResource(R.drawable.loadingcover);
				(new LoadCover()).execute(release.getMbid());
			}
			else
			{
				coverImageView.setImageDrawable(drawable);
			}
		}
		else
		{
			coverImageView.setVisibility(View.GONE);
		}
	}

	@Override
	public void cacheWidgets(View row)
	{
		dateTextView = (TextView) row.findViewById(R.id.date);
		releaseTextView = (TextView) row.findViewById(R.id.release);
		artistTextView = (TextView) row.findViewById(R.id.artist);
		coverImageView = (ImageView) row.findViewById(R.id.cover);
	}

	/* ************************************************************ */
	/*							COVERS            					*/
	/* ************************************************************ */

	private class LoadCover extends AsyncTask<String, Void, String>
	{
		private Drawable drawableCover;

		@Override
		protected String doInBackground(String... arg0)
		{
			try
			{
				drawableCover =  MuspyClient.getInstance().getCover(arg0[0]);
				CoverMemoryCache.add(arg0[0], drawableCover);
			}
			catch (Exception ex) 
			{
				Log.e(LoadCover.class.toString(), ex.getMessage(), ex);
			}

			return arg0[0];
		}

		@Override
		protected void onPostExecute(String result)
		{
			// es posible que la row que mostramos en pantalla cuando se termine
			// de ejecutar el hilo no sea
			// la que solicitó el cover porque la vista es reutilizada por
			// android y mientras se cargaba
			// la imagen el usuario ha hecho scroll,
			// en ese caso no la actualizamos porque el cover sería incorrecto,
			// Podemos identifcar la row que solicita cada hilo por el mbid
			// ya que se actualizará correctamente cuando se ejecute su hilo.
			// en cualquier caso, guardamos la imagen por si hacemos scroll
			// hacia atrás.
			if (mbid.equals(result))
			{
				coverImageView.setImageDrawable(drawableCover);
			}

		}

	}

}