/*
 * Copyright (C) 2012-2013 Daniel Medina <http://danielme.com>
 * 
 * This file is part of "Muspy for Android".
 * 
 * "Muspy for Android" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * "Muspy for Android" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with this program.  If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>
 */
package com.danielme.muspyforandroid.activities.base;

import java.util.Collection;

import android.content.SharedPreferences;
import android.preference.PreferenceManager;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.TextView;

import com.danielme.muspyforandroid.Constants;
import com.danielme.muspyforandroid.R;
import com.danielme.muspyforandroid.activities.adapters.template.CustomListAdapter;

/**
 * Custom ListActivity.
 * @author Daniel Medina (danielme.com)
 * @since 1.0
 *
 */
public abstract class AbstractListActivity extends AbstractActivity
{

	/* ************************************************************ */
	/*							FIELDS            					*/
	/* ************************************************************ */	
	
	private ListView listView;

	private CustomListAdapter customListAdapter;
	
	private AdapterView.OnItemClickListener mOnClickListener = new AdapterView.OnItemClickListener()
	{
		@SuppressWarnings("rawtypes")
		public void onItemClick(AdapterView parent, View view, int position, long id)
		{
			onListItemClick((ListView) parent, view, position, id);
		}
	};


	/* ************************************************************ */
	/*					        PROTECTED            			    */
	/* ************************************************************ */

	/**
	 * Updates the number of elements that shows the ListView.
	 */
	protected void updateDisplaying(int idString, String total)
	{
		TextView textView = (TextView) findViewById(R.id.displaying);
		String text = getString(idString);
		text = String.format(text, getCount(), total);
		textView.setText(text);
		textView.setVisibility(View.VISIBLE);
		if (getCount() > 0)
		{
			listView.setVisibility(View.VISIBLE);
			findViewById(R.id.background).setVisibility(View.GONE);
		}
		else
		{
			listView.setVisibility(View.GONE);
			findViewById(R.id.background).setVisibility(View.VISIBLE);
		}
	}

	protected CustomListAdapter getCustomListAdapter()
	{
		return customListAdapter;
	}

	protected ListView getListView()
	{
		if (listView == null)
		{
			initListView();
		}
		return listView;
	}

	protected void setListAdapter(ListAdapter adapter)
	{
		getListView().setAdapter(adapter);
		customListAdapter = (CustomListAdapter) adapter;
	}

	/**
	 * Returns the number of elements in this ListView's Adapter. Not null-safe.
	 */
	protected int getCount()
	{
		return customListAdapter.getCount();
	}


	protected void setSelection(int position)
	{
		getListView().setSelection(position);
	}

	protected int getSelectedItemPosition()
	{
		return getListView().getSelectedItemPosition();
	}

	/**
	 * Appends more objects to the adapter.
	 * @param objects
	 */
	protected void appendData(Collection<?> objects)
	{
		if (customListAdapter != null)
		{
		 customListAdapter.appendData(objects);
		}
	}

	/**
	 * Removes all of the elements in the Adapter.
	 */
	protected void clearData()
	{
		if (customListAdapter != null)
		{
		 customListAdapter.clearData();
		}
	}

	protected void onListItemClick(ListView listView, View view, int position, long id)
	{
		//optional
	}

	/**
	 * Returns the size of the page (for PaginatedListViews).
	 */
	protected int getListSize()
	{
		SharedPreferences defaultSharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
		String num = defaultSharedPreferences.getString(Constants.PREF_SIZE, "10");
		return Integer.parseInt(num);
	}
	
	public boolean showLast(int firstVisibleItem, int visibleItemCount, int totalItemCount)
	{
		return firstVisibleItem + visibleItemCount == totalItemCount && getListView().getChildAt(visibleItemCount - 1) != null 
				&& getListView().getChildAt(visibleItemCount - 1).getBottom() <= getListView().getHeight();
	}

	/* ************************************************************ */
	/*					         PRIVATE            			    */
	/* ************************************************************ */

	private void initListView()
	{
		listView = (ListView) findViewById(android.R.id.list);
		listView.setOnItemClickListener(mOnClickListener);
	}
}